-- Funcionamento:
--  O valor das teclas pressionados serão 
--  mostrados na Matriz de LEDs, no display 
--  de 7 segmentos e no LCD(LCD mostra os 
--  valores de forma continua).
--  Backlight é na chave SW3
--  Reset no botão Key(11)



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity Labx2 is
generic(
   DEBOUNCE    : integer := 2500000   -- filtro ruido de trepidaçao,2500000*20ns = 0,05s
   );
port(
		CLOCK_50MHz		: in std_logic;
		KEY				: in std_logic_vector (11 downto 0);
		LEDM_C			: out std_logic_vector(4 downto 0);
		LEDM_R			: out std_logic_vector(7 downto 0)
	);
end Labx2;

architecture behavior of Labx2 is

signal reset			   : std_logic;
signal key_data		   : std_logic_vector(3 downto 0);
signal key_data_reg	   : std_logic_vector(3 downto 0);
signal key_data_reg_w	: std_logic_vector(4 downto 0);
signal display_Q		   : std_logic_vector(6 downto 0);
signal en_display		   : std_logic;
signal display_off	   : std_logic;
signal ready_key		   : std_logic;

-- Declaraçao do teclado
component teclado_base is
generic(
	DEBOUNCE : integer
);
port(
		clk_i				:	in std_logic;
		push_button_i	:	in std_logic_vector (11 downto 0);
		key_o				: 	out std_logic_vector (3 downto 0);
		ready_o			:	out std_logic
	);
end component;

-- Declaraçao da Matriz de LEDs
component matriz_leds is
port(
		clk_i		:		in std_logic;
		rst_i		:		in std_logic;
		idx_i		: 		in std_logic_vector (4 downto 0);
		col_o		:		out std_logic_vector (4 downto 0);
		row_o		: 		out std_logic_vector (7 downto 0)
);
end component;
	
-- Declaração sincronizador de reset
COMPONENT reset_sync
   PORT
	(
		i_clk	               :	 IN STD_LOGIC;
		i_external_reset_n	:	 IN STD_LOGIC;
		o_reset_n      		:	 OUT STD_LOGIC;
		o_reset		         :	 OUT STD_LOGIC
   );
END COMPONENT;   
   
begin

teclado:
	teclado_base 
	generic map
	(
	DEBOUNCE => DEBOUNCE
	)
	port map
	(
		clk_i				=> CLOCK_50MHz,
		push_button_i 	=> '0'& KEY(10 downto 0),
		key_o				=> key_data,
		ready_o			=> ready_key
	);

	--Instaciaçao da matriz de Leds
	matriz_de_leds:	
	matriz_leds	port map (
		clk_i		=> CLOCK_50Mhz,
		rst_i		=> reset,
		idx_i		=> key_data_reg_w,
		col_o		=> LEDM_C,
		row_o		=> LEDM_R
	);
	
	-- Atribuiçao do KEY(11) como reset
   reset_synch_50mhz_inst : reset_sync
	PORT MAP (
		i_clk                => CLOCK_50MHz,
		i_external_reset_n   => KEY(11),
		o_reset		         => open,
		o_reset_n            => reset
	);
	--reset <= KEY(11); 
	-- Enable do registrador, para manter o dado do 
	en_display <= '1' when key_data /= "1111" else '0';

   
   --	Registro do teclado pra manter as saida estáveis
	process(CLOCK_50MHz, reset)
	begin
	if(reset = '1') then
		key_data_reg <= (others => '1');	 -- apagado em '1'
	elsif(rising_edge(CLOCK_50MHz)) then
		if en_display = '1'  and ready_key = '1' then
			key_data_reg <= key_data;
		end if;
	end if;
	end process;
   
   key_data_reg_w <= "01010" when (key_data_reg = "1111" or key_data_reg > "1010") else "0" & key_data_reg;
   
end behavior;	